#!/usr/bin/env python
# -*- coding: UTF-8 -*-

# Hält einen VPN-Tunnel offen, indem einmal pro Minute geprüft wird, ob der Tunnel noch steht.
# Der Aufruf erfolgt über die crontab mittels:    *  * * * *   $HOME/bin/tunnel_watchdog.py
#
#
# Ablauf ist (grob) folgender:
#     wenn die Außenwelt erreichbar ist
#       und die externe IP ungleich der externen Fritzbox-IP ist
#         dann ist alles gut...
#       wenn externe IP gleich der externen Fritzbox-IP ist
#         dann steht der Tunnel nicht mehr
#         Tunnel neu starten
#     wenn die Außenwelt nicht erreichbar ist
#       Info-Ausgabe: läuft openvpn, Default-GW
#       wenn openvpn läuft und Default-GW==tun0
#         etwas warten
#         Außenwelt-Test 3x wiederholen
#       wenn Außenwelt immer noch nicht erreichbar ist
#         Tunnel neu starten
#
#     wenn angeforderter Tunnel-Neustart nicht geklappt hat
#       reboot
#
#
# Benötigt wird: openvpn und whois
# Ebenfalls benötigt (sollten aber per Default da sein): route, ping, ps, pidof, reboot
#
# openvpn, route, kill und reboot müssen per sudo ohne Passworteingabe ausführbar sein.
#
# Für Python wird "python-pycurl" benötigt.
#
# D.A. 04/05.2014
#
# 24.05.2014: optionale Transmission-Überwachung zugefügt
# 03.07.2014: check_default_gw() auf "route -n" umgestellt (für CubieTruck)
# 31.07.2014: bwm2responder.py-Überwachung zugefügt
# 31.10.2014: Fix für FRITZ!OS 06.20 eingebaut
#

import os
import sys
import subprocess
import socket
import time
import datetime
import threading
import pycurl
import StringIO

# Dateien im Homeverzeichnis
LOGFILE=os.path.join(os.path.expanduser('~'), "log_tunnel_watchdog.txt")
CFGFILE=os.path.join(os.path.expanduser('~'), ".tunnel_watchdog.conf")

# Daten der Fritzbox
FRITZBOX_IP   ="192.168.178.1"
FRITZBOX_PORT =49000

# Diese IPs (GoogleDNS, OpenDNS, dns.as250.net) werden für den Außenwelt-Test verwendet
PING_SERVERS  =["8.8.8.8", "8.8.4.4", "208.67.222.222", "208.67.220.220", "194.150.168.168"]

AUTO_RECONNECT_AT="04:30"  # Um 04:30 Uhr den Tunnel neu aufbauen


TRANSMISSION_MONITORING=False
PATH_TRANSMISSION="/usr/bin/transmission-daemon"
PATH_TORRENTCYCLE="/home/dede/bin/torrentcycle.py"

PATH_BWM2RESPONDER="/home/dede/bin/bwm2responder.py"

# ###########################################################
# Fragt bei diversen Diensten bzgl. der eigenen IP an und
# liefert diese als Liste von [mehreren hoffentlich gleichen]
# IP-Strings mit getAllIPs(). Bei waitForIP() wird die zuerst
# gelieferte IP zurückgegeben, oder "" wenn keine innerhalb
# der Timeout-Zeit erkannt wurde.
# (braucht unter Debian das Paket "python-pycurl")
class GetExternalIP():
  def __init__(self):
    self.ip_list=[]
    self.url_list=[ "http://ipecho.net/plain", 
                    "http://www.networksecuritytoolkit.org/nst/cgi-bin/ip.cgi",
                    "v4.ident.me",
                    "ipv4.icanhazip.com",
                    "ipv4.ipogre.com"
                  ]
    for i in range(len(self.url_list)):  # Worker aufsetzen
      worker=threading.Thread(target=self.__processURL, name="processURL("+str(i)+")")
      worker.setDaemon(True)
      worker.start()

  # ###########################################################
  # Liefert die zuerst gelieferte IP zurück, sofern diese in
  # "wait_in_sec" Sekunden ankommt. Sonst wird "" geliefert.
  def waitForIP(self, wait_in_sec):
    cnt=wait_in_sec*10
    while self.ip_list==[]: # solange keine IP geliefert wurde...
      time.sleep(0.1)       # warten
      cnt-=1
      if cnt<=0:            # Timeout
        return("")
    return(self.ip_list[0])

  # ###########################################################
  # Liefert alle bisher angekommenen IPs als Liste von Strings.
  def getAllIPs(self):
    return(self.ip_list)

  # ###########################################################
  # Holt eine URL von der Liste, fragt dort an und hängt
  # das Ergebnis im Erfolgsfalle "self.ip_list" an.
  def __processURL(self):
    strg=StringIO.StringIO()
    curl=pycurl.Curl()
    url=self.url_list.pop()                       # URL von der Liste nehmen
    curl.setopt(pycurl.NOSIGNAL, 1)               # Workaround für bekannten Bug von pycurl
    curl.setopt(pycurl.URL, url)                  # URL übergeben
    curl.setopt(pycurl.WRITEFUNCTION, strg.write) # die DatenSammelFunktion übergeben
    curl.setopt(pycurl.CONNECTTIMEOUT, 5)         # connection-phase-Timeout setzen
    curl.setopt(pycurl.TIMEOUT, 5)                # transfer-Timeout setzen
    curl.setopt(pycurl.USERAGENT, "curl")         # nur IP statt komplette WebSeite anfordern
    try:
      curl.perform()                              # Abfrage ausführen
    except:
      return
    ip=strg.getvalue().strip()                    # Ergebnis merken
    curl.close()
    strg.close()
    if self.is_valid_ip(ip)==True:                # wenn legale IP-Adresse...
      self.ip_list.append(ip)                     # ...IP übergeben

  # ###########################################################
  # Liefert True, wenn "ip" eine syntaktisch korrekte
  # IP-Adresse darstellt.
  def is_valid_ip(self, ip):
    if ip.count(".")!=3:
      return(False)     # IP enthält nicht genau drei Punkte
    for c in ip:
      if c not in "0123456789.":
        return(False)   # IP enthält illegales Zeichen
    for octet in ip.split("."):
      try:
        octet_num=int(octet)
      except:
        return(False)   # Oktett ist nicht numerisch
      if octet_num<0 or octet_num>255:
        return(False)   # Oktett enthält illegalen numerischen Wert
    return(True)




# ###########################################################
# Liefert Daten von der Fritzbox.
class Fritzbox():
  def __init__(self, host=FRITZBOX_IP, port=FRITZBOX_PORT):
    self.host=host
    self.port=port

    self.upnpControl="upnp/control/"
    s=self.__send_req_resp(self.upnpControl+"WANIPConn1", "WANIPConnection:1", "GetExternalIPAddress")
    if s.find("<faultstring>UPnPError</faultstring>")>=0:
      self.upnpControl="igdupnp/control/"  # Fix für >= FRITZ!OS 06.20

  # ###########################################################
  # Sendet eine Anfrage an die Fritzbox und liefert den
  # Antwortstring zurueck
  def __send_req_resp(self, controlURL, serviceType, action):
    body='<?xml version="1.0"?>\n' \
         '<s:Envelope xmlns:s="http://schemas.xmlsoap.org/soap/envelope/"\n' \
         ' s:encodingStyle="http://schemas.xmlsoap.org/soap/encoding/">\n' \
         '  <s:Body>\n' \
         '    <u:'+action+' xmlns:u="urn:schemas-upnp-org:service:'+serviceType+'"/>\n' \
         '  </s:Body>\n' \
         '</s:Envelope>\n'
    pream='POST /'+controlURL+' HTTP/1.0\r\n' \
          'HOST: '+self.host+':'+str(self.port)+'\r\n' \
          'CONTENT-LENGTH: '+str(len(body))+'\r\n' \
          'CONTENT-TYPE: text/xml; charset="utf-8"\r\n' \
          'SOAPACTION: "urn:schemas-upnp-org:service:'+serviceType+'#'+action+'"\r\n\r\n'
    sock=socket.socket(socket.AF_INET, socket.SOCK_STREAM)
    sock.connect((self.host, self.port))
    sock.send(pream + body)

    resp=""
    while True:
      data=sock.recv(1024)
      if len(data)==0:
        break
      resp+=data
    sock.close()
    return(resp)

  # ###########################################################
  # liefert aus "xml_str" den Teilstring, der zwischen
  #  <"arg"> und </"arg"> steht.
  def __get_argument(self, xml_str, arg):
    v="<"+arg+">"
    p1=xml_str.find(v)
    if p1>=0:
      p2=xml_str.find("</"+arg+">", p1)
      if p2>=0:
        return(xml_str[p1+len(v):p2])
    return(None)

  # ###########################################################
  # Liefert die externe IP der Fritzbox.
  def getExternalIP(self):
    s=self.__send_req_resp(self.upnpControl+"WANIPConn1", "WANIPConnection:1", "GetExternalIPAddress")
    ip=self.__get_argument(s, "NewExternalIPAddress")
    return(ip)

  # ###########################################################
  # Liefert den DNS-Server der Fritzbox.
  def getDNS(self):
    s=self.__send_req_resp(self.upnpControl+"WANCommonIFC1", "WANCommonInterfaceConfig:1", "GetAddonInfos")
    dns=self.__get_argument(s, "NewDNSServer1")
    return(dns)




# ###########################################################
# Verwaltung einer einfachen KeyValue-Datei.
class ConfigFile():
  # ###########################################################
  # Lädt die Datei "filename" in das Dictionary "self.cfgDict".
  def __init__(self, filename):
    self.filename=filename
    self.fobj=self.__open("r")
    if self.fobj!=None:
      lns=self.fobj.readlines()
      self.cfgDict=self.__stringListToDict(lns)
      self.fobj.close()
    else:
      self.cfgDict={}

  # ###########################################################
  # Liefert den Wert des Attributes "attr". Wenn das Attribut
  # nicht existiert, wird "noAttrVal" geliefert.
  def getAttr(self, attr, noAttrVal=None):
    return(self.cfgDict.get(attr, noAttrVal))

  # ###########################################################
  # Setzt das Attribut "attr" auf den Wert "val". Sofern
  # "writeToDisk" nicht auf False gesetzt wird, wird die
  # Datei danach neu auf Platte geschrieben.
  def setAttr(self, attr, val, writeToDisk=True):
    self.cfgDict.update({attr.upper() : val})
    if writeToDisk==True:
      self.fobj=self.__open("w")
      if self.fobj!=None:
        for a, v in self.cfgDict.items():
          self.fobj.write(a+"="+v+"\n")
        self.fobj.close()

  # ###########################################################
  # Wandelt eine Liste aus Strings im Format
  #    ["aa=bb", "cc=dd\n", ...]
  # in ein entspr. Dictionary im Format
  #    {"AA":"bb", "CC":"dd", ...}.
  def __stringListToDict(self, stringList):
    d={}
    for l in stringList:
      if l.find("=")>0:
        a, v=l.split("=", 1)
        d.update({a.upper() : v.rstrip()})
    return(d)

  # ###########################################################
  # Öffnet die Datei "self.filename" in dem Modus "mode".
  # Liefert das entspr. Dateiobjekt oder None, wenn die Datei
  # nicht geöffnet werden konnte.
  def __open(self, mode):
    try:
      fobj=open(self.filename, mode)
    except:
      fobj=None
    return(fobj)




# ###########################################################
# Schreibt "txt" in die Datei LOGFILE.
def logStatus(txt):
  fobj=open(LOGFILE, "a")
  fobj.write(time.strftime("%Y.%m.%d-%H:%M:%S")+" "+txt+"\n")
  fobj.close()



# ###########################################################
# Führt "cmd" aus und liefert dessen Rückgabe 
# bzw. "" bei Fehler.
def executeCmd(cmd):
  try:
    lns=subprocess.check_output(cmd, stderr=open(os.devnull, 'w'))
  except:
    return("")
  return(lns)



# ###########################################################
# Führt "cmd" aus und liefert die erste Zeile aus der
# Rückgabe, in der "keyword" vorkommt. Oder "", wenn nicht.
def findInCmd(cmd, keyword):
  lns=executeCmd(cmd)
  for l in lns.split("\n"):
    if l.find(keyword)>=0:
      return(l)
  return("")



# ###########################################################
# Liefert die Länderkennung zu "ip" gemäß <whois> oder "??",
# wenn keine Länderkennung ermittelt werden konnte.
# Erwartet von "whois" sowas wie:
#   country:        NL
def getIPCountry(ip):
  rc=findInCmd(["whois", ip], "country:").split()
  if len(rc)>1:
    return(rc[1].upper())
  return("??")



# ###########################################################
# Liefert True, wenn openvpn läuft, sonst False.
# Erwartet von "ps aux" sowas wie:
#   root     22281  0.1  0.3  29048  3164 ?        Ss   14:54   0:08 /usr/sbin/openvpn ....
def check_openvpn():
  if findInCmd(["ps", "aux"], "/usr/sbin/openvpn")!="":
    return(True)
  return(False)
# ###########################################################
# Liefert True, wenn transmission läuft, sonst False.
def check_transmission():
  if findInCmd(["ps", "aux"], PATH_TRANSMISSION)!="":
    return(True)
  return(False)
# ###########################################################
# Liefert True, wenn torrentcycle.py läuft, sonst False.
def check_torrentcycle():
  if findInCmd(["ps", "aux"], PATH_TORRENTCYCLE)!="":
    return(True)
  return(False)
# ###########################################################
# Liefert True, wenn bwm2responder.py läuft, sonst False.
def check_bwm2responder():
  if findInCmd(["ps", "aux"], PATH_BWM2RESPONDER)!="":
    return(True)
  return(False)




# ###########################################################
# Liefert True, wenn das Default Gateway auf dem Interface
# "tun0" liegt. Sonst False.
# Erwartet von "sudo /sbin/route -n" sowas wie:
#   0.0.0.0         10.3.129.221    0.0.0.0         UG    0      0        0 tun0
def check_default_gw():
  lns=executeCmd(["sudo", "/sbin/route", "-n"]).split("\n")
  for l in lns:
    ls=l.split()
    if len(ls)>4:
      if ls[0]==ls[2]=="0.0.0.0":
        if ls[len(ls)-1]=="tun0":
          return(True)
  return(False)



# ###########################################################
# Liefert True, wenn mindestes eine der IPs aus PING_SERVERS
# nach maximal einer Sekunde auf ping antwortet. Sonst False.
def check_world():
  for ip in PING_SERVERS:
    if executeCmd(["ping", "-W", "1", "-c", "1", ip])!="":
      return(True)
  return(False)



# ###########################################################
# Liefert den Zeitpunkt des letzten Reboots als Datetime-Objekt
# im ersten Rückgabewert. Der zweite Rückgabewert ist die
# verwendete aktuelle Zeit als Datetime-Objekt (zwecks Referenz).
def getRebootTime():
  with open('/proc/uptime', 'r') as f:
    uptimeSeconds=float(f.readline().split()[0])
  nowTime=datetime.datetime.now()
  rebootTime=nowTime-datetime.timedelta(seconds=uptimeSeconds)
  return((rebootTime.replace(microsecond=0), nowTime.replace(microsecond=0)))



# ###########################################################
# Startet openvpn neu durch.
# Liefert False, wenn "stop" oder "start" nicht klappt oder
# das DefaultGW am Schluß nicht auf tun0 steht. Sonst True.
def restart_tunnel():
  maxwait=10

  wait=maxwait
  executeCmd(["sudo", "service", "openvpn", "stop"])
  logStatus("stoppe openvpn")
  while check_openvpn()==True:  # auf STOP warten
    time.sleep(1)
    wait-=1
    if wait<=2:         # lange genug gewartet - jetzt wird gekillt
      logStatus("töte openvpn")
      pidlst=executeCmd(["pidof", "/usr/sbin/openvpn"]).strip().split()
      logStatus("pid="+"/".join(pidlst))
      for pid in pidlst:
        executeCmd(["sudo", "kill", "-9", pid])
    else:
      logStatus("stoppe openvpn")
    if wait<=0:
      return(False)         # der STOP hat offenbar nicht geklappt...
  logStatus("openvpn gestoppt")

  time.sleep(3)

  wait=maxwait
  executeCmd(["sudo", "service", "openvpn", "start"])
  logStatus("starte openvpn")
  while check_openvpn()==False: # auf START warten
    time.sleep(1)
    wait-=1
    if wait<=0:
      return(False)         # der START hat offenbar nicht geklappt...
    logStatus("starte openvpn")
  logStatus("openvpn gestartet")

  wait=maxwait
  while check_default_gw()==False:  # auf "default" = "tun0" warten
    logStatus("warte auf tun0")
    time.sleep(1)
    wait-=1
    if wait<=0:
      return(False)         # tun0 kommt nicht hoch...
  logStatus("tun0 steht")
  return(True)



# ###########################################################
# Startet transmission-daemon neu durch.
def restart_transmission():
  maxwait=10

  wait=maxwait
  executeCmd(["sudo", "service", "transmission-daemon", "stop"])
  logStatus("stoppe transmission")
  while check_transmission()==True:  # auf STOP warten
    time.sleep(1)
    wait-=1
    if wait<=2:         # lange genug gewartet - jetzt wird gekillt
      logStatus("töte transmission")
#      pidlst=executeCmd(["pidof", "/usr/bin/transmission-daemon"]).strip().split()
      pidlst=executeCmd(["pidof", PATH_TRANSMISSION]).strip().split()
      logStatus("pid="+"/".join(pidlst))
      for pid in pidlst:
        executeCmd(["sudo", "kill", "-9", pid])
    else:
      logStatus("stoppe transmission")
    if wait<=0:
      return(False)         # der STOP hat offenbar nicht geklappt...
  logStatus("transmission gestoppt")

  time.sleep(3)

  wait=maxwait
  executeCmd(["sudo", "service", "transmission-daemon", "start"])
  logStatus("starte transmission")
  while check_transmission()==False: # auf START warten
    time.sleep(1)
    wait-=1
    if wait<=0:
      return(False)         # der START hat offenbar nicht geklappt...
    logStatus("starte transmission")
  logStatus("transmission gestartet")
  return(True)



# ###########################################################
# Die Haupt-Schleife.
if __name__=="__main__":
  tunnel_is_ok=None
  cfgFile=ConfigFile(CFGFILE)

  if TRANSMISSION_MONITORING==True:
    if check_torrentcycle()==False:
      # torrentcycle.py läuft nicht
      os.system("nohup %s &"%PATH_TORRENTCYCLE)
      logStatus("torrentcycle.py wurde gestartet!")

#  if check_bwm2responder()==False:
#    logStatus("bwm2responder.py wurde gestartet!")
#    os.system("nohup %s &"%PATH_BWM2RESPONDER)

  lrca_str=cfgFile.getAttr("LAST_RECONNECT_AT")
  if lrca_str!=None:
    # Wenn die Zeit des letzen Reboots jünger ist, als die Zeit des letzten
    # openvpn-Starts, dann muss es einen Reboot gegeben haben.
    # In diesem Fall ist die Reboot-Zeit als neue openvpn-Startzeit zu setzen.
    lrca=datetime.datetime.strptime(lrca_str, "%Y.%m.%d-%H:%M:%S")
    lrba=getRebootTime()[0]
    if lrba>lrca: # wenn Reboot-Zeit jünger als openvpn-Startzeit...
      lrba=lrba+datetime.timedelta(seconds=2) # openvpn-Startzeit = Reboot-Zeit + 2 Sek.
      lrba_str=lrba.strftime("%Y.%m.%d-%H:%M:%S") # Reboot-Zeit nach String
      cfgFile.setAttr("LAST_RECONNECT_AT", lrba_str, writeToDisk=False)
      logStatus("Wert von LAST_RECONNECT_AT wurde korrigiert!")
      if TRANSMISSION_MONITORING==True:
        restart_transmission()

  if AUTO_RECONNECT_AT==time.strftime("%H:%M"):
    # es ist Zeit für den geplanten Tunnel-Reconnect
    logStatus("Der Tunnel wird geplant neu aufgebaut!")
    tunnel_is_ok=False
  elif check_world()==True:
    # die Außenwelt ist erreichbar
    ips=GetExternalIP()   # asynchrones IP holen anstoßen
    ip=ips.waitForIP(5)   # maximal 5 Sekunden lang auf IP warten
    if ip!="":            # es wurde eine externe IP geliefert
      if int(cfgFile.getAttr("NO_EXT_IP_CNT", "0"))>=2:
        # wenn NO_EXT_IP_CNT auf mind. 2 steht, wurde beim letzten Durchlauf
        # eine entspr. Warnung ausgegeben. Daher jetzt Gut-Meldung ausgeben.
        logStatus("Externe IP konnte ermittelt werden!")
      cfgFile.setAttr("NO_EXT_IP_CNT", "0", writeToDisk=False)  # Zähler zurücksetzen
      letzte_ip=cfgFile.getAttr("LAST_GOOD_IP")
      if letzte_ip==ip:
        # externe IP ist unverändert zum letzten Durchlauf
        tunnel_is_ok=True
      else:
        # externe IP hat sich seit dem letzten Durchlauf geändert
        fb=Fritzbox()
        fritzboxIP=fb.getExternalIP() # externe IP der Fritzbox holen
        if ip!=fritzboxIP:
          # externe IP ungleich der externen Fritzbox-IP
          country_ip=getIPCountry(ip)
          #logStatus("Tunnel nach "+country_ip+" steht!  extIP="+ip+"  FbIP="+fritzboxIP+"\n")
          logStatus("Tunnel nach %s steht!  extIP=%s  FbIP=%s\n"%(country_ip, ip, fritzboxIP))
          cfgFile.setAttr("LAST_GOOD_IP", ip, writeToDisk=False)
          cfgFile.setAttr("LAST_GOOD_COUNTRY", country_ip, writeToDisk=False)
          tunnel_is_ok=True
        else:
          # die VM geht direkt über die Fritzbox raus
          logStatus("Der Tunnel steht offenbar nicht mehr!")
          tunnel_is_ok=False
    else:
      # Welt ist erreichbar, aber es konnte keine externe IP ermittelt werden
      cntNoExtIP=int(cfgFile.getAttr("NO_EXT_IP_CNT", "0"))+1 # Zähler inkrementieren
      if cntNoExtIP>=2:
        #logStatus("Externe IP konnte "+str(cntNoExtIP)+"x in Folge nicht ermittelt werden!")
        logStatus("Externe IP konnte %dx in Folge nicht ermittelt werden!"%(cntNoExtIP))
      if cntNoExtIP>4:
        tunnel_is_ok=False  # nun wirds aber langsam unheimlich -> Tunnel neu aufbauen
        # NO_EXT_IP_CNT wird unten eh auf "0" gesetzt und muss daher hier nicht geschrieben werden
      else:
        cfgFile.setAttr("NO_EXT_IP_CNT", str(cntNoExtIP), writeToDisk=False)
        tunnel_is_ok=True
  else:
    # die Außenwelt ist nicht erreichbar
    logStatus("Die Außenwelt ist nicht erreichbar!")
    scovpn=check_openvpn()
    scdgw=check_default_gw()
    logStatus("openvpn hoch:"+str(scovpn)+" defaultGW==tun0:"+str(scdgw))
    if scovpn==True and scdgw==True:
      # openvpn läuft und Default-GW ist gleich "tun0"
      # hier ist alles gut...die Außenseite hat ein Problem
      local_tunnel_ok=True
      doublecheck_success=False
      logStatus("Wiederholung des Außenwelt-Erreichbarkeits-Tests")
      for i in range(3):  # Außenwelt-Test 3x wiederholen (max. 18 Sek. bei 5 Pings)
        time.sleep(1)
        if check_world()==True:
          doublecheck_success=True
          break
    else:
      # openvpn läuft nicht oder Default-GW ist ungleich "tun0"
      local_tunnel_ok=False

    if local_tunnel_ok==False or doublecheck_success==False:
      # Tunnel steht nicht oder Außenwelt ist immer noch nicht erreichbar
      tunnel_is_ok=False
    else:
      # Tunnel steht und Außenwelt ist erreichbar
      logStatus("Die Außenwelt ist erreichbar")
      tunnel_is_ok=True

  if TRANSMISSION_MONITORING==True:
    if tunnel_is_ok==True:                # wenn Tunnel steht...
      if check_transmission()==False:     # ...aber Transmission nicht hoch ist...
        if restart_transmission()==False: # ...Transmission neu starten
          tunnel_is_ok=False              # wenn das nicht geklappt hat -> Tunnel neu starten

  if tunnel_is_ok==True:
    cfgFile.setAttr("LAST_RUN_AT", time.strftime("%Y.%m.%d-%H:%M:%S"))
    # den Interpreter am Laufen halten, bis die Threads in GetExternalIP() entweder
    # regulär Daten geliefert haben oder der Timeout greifen konnte
    time.sleep(10)
  elif tunnel_is_ok==False:
    cfgFile.setAttr("LAST_GOOD_IP", "?.?.?.?", writeToDisk=False)
    logStatus("Tunnel wird neu aufgebaut!")
    if restart_tunnel()==True:
      cfgFile.setAttr("NO_EXT_IP_CNT", "0", writeToDisk=False)
      cfgFile.setAttr("LAST_RECONNECT_AT", time.strftime("%Y.%m.%d-%H:%M:%S"))
      logStatus("Tunnel wurde aufgebaut")
      if TRANSMISSION_MONITORING==True:
        restart_transmission()  # neu starten, damit Transmission den "Listening port" neu öffnet
    else:
      cfgFile.setAttr("NO_EXT_IP_CNT", "0")
      logStatus("Tunnel wurde nicht korrekt aufgebaut -> Reboot")
      executeCmd(["sudo", "reboot"])

